/**************************************************************************
 * Version: Simplified BSD License
 * URL: http://www.opensource.org/licenses/bsd-license.php
 *
 * Copyright (c) 2010, Joseph LaFata
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - Neither the name of cppbench nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#ifndef BENCHFORMATTER_H_
#define BENCHFORMATTER_H_

#include "DllExport.h"

#include "BenchResult.h"

/** @brief An abstract class for writing benchmark information.
 *
 * Implement the functions in this abstract class to provide
 * output from the benchmarks.
 */
class CPPBENCH_API BenchFormatter
{
public:
	BenchFormatter();
	virtual ~BenchFormatter();


    /** @brief write the titles of the columns or groups.
     *
     * @param titles std::vector<std::string>&.
     *
     */
	virtual void writeTitle(const std::vector<std::string>& titles,
			const std::vector<std::string>& values) = 0;

    /** @brief write the result information for value val.
     *
     * @param val const std::string& the value.
     *
     * @param results std::vector<BenchResult> a vector of benchmark results.
     *
     * The benchmark results appear in the order the corresponding benchmarks
     * were added to the BenchmarkGroup.
     *
     */
	virtual void writeResult(const std::string& val, std::vector<BenchResult> results) = 0;

    /** @brief Called after all benchmarks have run.
     *
     * This is a good place to close any open file handles.
     */
	virtual void finish() { };
};

#endif /* BENCHFORMATTER_H_ */
