/**************************************************************************
 * Version: Simplified BSD License
 * URL: http://www.opensource.org/licenses/bsd-license.php
 *
 * Copyright (c) 2010, Joseph LaFata
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - Neither the name of cppbench nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include "BenchResult.h"

#include "float.h"
#include "math.h"

BenchResult::BenchResult()
{

}

BenchResult::BenchResult(std::vector<Stopwatch::Time>& times)
{
	double defaults[TimeValue::LAST] = {0.0, DBL_MAX, DBL_MIN, 0.0};
	for (unsigned i = 0; i < TimeType::LAST; i++)
	{
		for (unsigned j = 0; j < TimeValue::LAST; j++)
		{
			timeValues[i][j] = defaults[j];
		}
	}

    for (unsigned i = 0; i < times.size(); i++)
    {
		timeValues[TimeType::WALL_CLOCK][TimeValue::AVERAGE] += times[i].wall;
		timeValues[TimeType::USER][TimeValue::AVERAGE] += times[i].user;
		timeValues[TimeType::SYSTEM][TimeValue::AVERAGE] += times[i].sys;

		timeValues[TimeType::WALL_CLOCK][TimeValue::MINIMUM] 
			= std::min(timeValues[TimeType::WALL_CLOCK][TimeValue::MINIMUM], times[i].wall);
		timeValues[TimeType::USER][TimeValue::MINIMUM] 
			= std::min(timeValues[TimeType::USER][TimeValue::MINIMUM], times[i].user);
		timeValues[TimeType::SYSTEM][TimeValue::MINIMUM] 
			= std::min(timeValues[TimeType::SYSTEM][TimeValue::MINIMUM], times[i].sys);

		timeValues[TimeType::WALL_CLOCK][TimeValue::MAXIMUM] 
			= std::max(timeValues[TimeType::WALL_CLOCK][TimeValue::MAXIMUM], times[i].wall);
		timeValues[TimeType::USER][TimeValue::MAXIMUM] 
			= std::max(timeValues[TimeType::USER][TimeValue::MAXIMUM], times[i].user);
		timeValues[TimeType::SYSTEM][TimeValue::MAXIMUM] 
			= std::max(timeValues[TimeType::SYSTEM][TimeValue::MAXIMUM], times[i].sys);
    }

	for (unsigned i = 0; i < TimeType::LAST; i++)
	{
		timeValues[i][TimeValue::AVERAGE] /= (double)times.size();
	}

	for (unsigned i = 0; i < times.size(); i++)
	{
		timeValues[TimeType::WALL_CLOCK][TimeValue::STD_DEV] += 
			pow((times[i].wall - timeValues[TimeType::WALL_CLOCK][TimeValue::AVERAGE]), 2);

		timeValues[TimeType::USER][TimeValue::STD_DEV] += 
			pow((times[i].user - timeValues[TimeType::USER][TimeValue::AVERAGE]), 2);
		
		timeValues[TimeType::SYSTEM][TimeValue::STD_DEV] += 
			pow((times[i].sys - timeValues[TimeType::SYSTEM][TimeValue::AVERAGE]), 2);
	}

	for (unsigned i = 0; i < TimeType::LAST; i++)
	{
		timeValues[i][TimeValue::STD_DEV] = 
			sqrt(timeValues[i][TimeValue::STD_DEV] / (double)times.size());
	}
}

BenchResult::~BenchResult()
{

}

void BenchResult::setTimeValue(TimeType::Type type, TimeValue::Value valueType, double value)
{ 
	timeValues[type][valueType] = value; 
}

double BenchResult::getTimeValue(TimeType::Type type, TimeValue::Value valueType) const
{
	return timeValues[type][valueType];
}
