/**************************************************************************
 * Version: Simplified BSD License
 * URL: http://www.opensource.org/licenses/bsd-license.php
 *
 * Copyright (c) 2010, Joseph LaFata
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - Neither the name of cppbench nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ***************************************************************************/

#include "ConsoleFormatter.h"

void ConsoleFormatter::writeTitle(const std::vector<std::string>& titles,
			const std::vector<std::string>& values)
{
	mFirstColumn = 6;
	for (size_t i = 0; i < values.size(); i++)
	{
		mFirstColumn = std::max(values[i].size()+1, (size_t)mFirstColumn);
	}
	std::cout << std::setw(mFirstColumn) << std::left << "Value";

	for (size_t i = 0; i < titles.size(); i++)
	{
		size_t col = std::max(titles[i].size()+1, (size_t)11);
		mColSize.push_back(col);
		std::cout << std::setw(col) << std::left << titles[i];
	}

	std::cout << std::endl;
}

void ConsoleFormatter::writeResult(const std::string& val, std::vector<BenchResult> results)
{
	std::cout << std::setw(mFirstColumn) << val;

	for (size_t i = 0; i < results.size(); i++)
	{
		std::cout << std::setiosflags(std::ios::fixed)
			<< std::setw(mColSize[i]) <<
			std::left << std::setprecision(8) << results[i].getTimeValue(TimeType::WALL_CLOCK, TimeValue::AVERAGE);
	}

	std::cout << std::endl;
}
